@@ -44,22 +44,29 @@ place_labels <- function(rects, polygons, bounds, anchors, ghosts) {
 #' @importFrom grid convertWidth convertHeight nullGrob polylineGrob
 #' @importFrom stats runif
 make_label <- function(labels, dims, polygons, ghosts, buffer, con_type,
-                       con_border, con_cap, con_gp, anchor_mod, arrow) {
-    polygons <- lapply(polygons, function(p) {
-        if (length(p$x) == 1 & length(p$y) == 1) {
-            list(
-                x = runif(200, p$x-0.00005, p$x+0.00005),
-                y = runif(200, p$y-0.00005, p$y+0.00005)
-            )
-        } else {
-            list(
-                x = p$x,
-                y = p$y
-            )
-        }
-    })
+                       con_border, con_cap, con_gp, anchor_mod, anchor_x,
+                       anchor_y, arrow) {
+  polygons <- lapply(polygons, function(p) {
+    if (length(p$x) == 1 & length(p$y) == 1) {
+      list(
+        x = runif(200, p$x-0.00005, p$x+0.00005),
+        y = runif(200, p$y-0.00005, p$y+0.00005)
+      )
+    } else {
+      list(
+        x = p$x,
+        y = p$y
+      )
+    }
+  })
 
-  anchors <- lapply(polygons, function(p) c(mean(range(p$x)), mean(range(p$y))))
+  anchors <- lapply(seq_along(polygons), function(i) {
+    x <- mean(range(polygons[[i]]$x))
+    if (length(anchor_x) == length(polygons) && !is.na(anchor_x[i])) x <- anchor_x[i]
+    y <- mean(range(polygons[[i]]$y))
+    if (length(anchor_y) == length(polygons) && !is.na(anchor_y[i])) y <- anchor_y[i]
+    c(x, y)
+  })
   p_big <- polyoffset(polygons, convertWidth(buffer, 'mm', TRUE))
 
   area <- c(
@@ -76,7 +83,7 @@ make_label <- function(labels, dims, polygons, ghosts, buffer, con_type,
     lab$vp$y <- unit(pos[2], 'mm')
     lab
   }, lab = labels, pos = labelpos)
-  connect <- do.call(rbind, Map(function(pol, pos, dim) {
+  connect <- inject(rbind(!!!Map(function(pol, pos, dim) {
     if (is.null(pos)) return(NULL)
     dim <- dim / anchor_mod
     pos <- cbind(
@@ -85,9 +92,9 @@ make_label <- function(labels, dims, polygons, ghosts, buffer, con_type,
     )
     pos <- points_to_path(pos, list(cbind(pol$x, pol$y)), TRUE)
     pos$projection[which.min(pos$distance), ]
-  }, pol = polygons, pos = labelpos, dim = dims))
-  labeldims <- do.call(rbind, dims[lengths(labelpos) != 0]) / 2
-  labelpos <- do.call(rbind, labelpos)
+  }, pol = polygons, pos = labelpos, dim = dims)))
+  labeldims <- inject(rbind(!!!dims[lengths(labelpos) != 0])) / 2
+  labelpos <- inject(rbind(!!!labelpos))
   if (con_type == 'none' || !con_type %in% c('elbow', 'straight')) {
     connect <- nullGrob()
   } else {
@@ -167,7 +174,7 @@ labelboxGrob <- function(label, x = unit(0.5, 'npc'), y = unit(0.5, 'npc'),
   sep_height <- if (lab_height > 0 && desc_height > 0) {
     pad[1]
   } else if (lab_height > 0) {
-    as_mm(grobDescent(lab_grob))
+    font_descent(gps$lab$fontfamily, gps$lab$fontface, gps$lab$fontsize, gps$lab$cex)
   } else {
     0
   }
@@ -282,7 +289,7 @@ elbow <- function(xmin, xmax, ymin, ymax, x, y) {
   end_angle <- atan2(end_pos[, 2], end_pos[, 1]) %% (2 * pi)
   angle_bin <- end_angle %/% (pi / 4)
   angle_lower <- end_angle %% (pi / 4) < 0.5
-  elbow <- do.call(rbind, lapply(seq_along(angle_bin), function(i) {
+  elbow <- lapply(seq_along(angle_bin), function(i) {
     a_bin <- angle_bin[i]
     a_lower <- angle_lower[i]
     if (a_bin == 0 || a_bin == 4) {
@@ -310,7 +317,8 @@ elbow <- function(xmin, xmax, ymin, ymax, x, y) {
         c(end_pos[i, 1] + end_pos[i, 2], 0)
       }
     }
-  }))
+  })
+  elbow <- inject(rbind(!!!elbow))
   elbow <- elbow + lines[[2]]
   colnames(elbow) <- c('x', 'y')
   list(lines[[1]], elbow, lines[[2]])
@@ -332,7 +340,7 @@ end_cap <- function(lines, cap) {
 zip_points <- function(points) {
   n_lines <- nrow(points[[1]])
   n_joints <- length(points)
-  points <- as.data.frame(do.call(rbind, points))
+  points <- as.data.frame(inject(rbind(!!!points)))
   points$id <- rep(seq_len(n_lines), n_joints)
   points[order(points$id), ]
 }
@@ -377,9 +385,16 @@ get_end_points <- function(xmin, xmax, ymin, ymax, x, y) {
     left = ifelse(abs(ymin_tmp) < abs(ymax_tmp), ymin, ymax),
     right = ifelse(abs(ymin_tmp) < abs(ymax_tmp), ymin, ymax)
   )
-  data.frame(x = x_new, y = y_new)
+  data_frame0(x = x_new, y = y_new)
 }
 vswitch <- function(x, ...) {
   cases <- cbind(...)
   cases[cbind(seq_along(x), match(x, colnames(cases)))]
 }
+
+font_descent <- function(fontfamily, fontface, fontsize, cex) {
+  italic <- fontface >= 3
+  bold <- fontface == 2 | fontface == 4
+  info <- systemfonts::font_info(fontfamily, italic, bold, fontsize * (cex %||% 1), res = 300)
+  as_mm(abs(info$max_descend)*72/300, 'pt', FALSE)
+}
